from os import path

class ConfigPath(object):
    def __init__(self):
        self.config_paths = [path.join(path.expanduser('~'), '.magpie'), path.dirname(__file__)]
    def __getattr__(self, key):
        for path in self.config_paths:
            return_path = path.join(path, key + '.cfg')
            if path.exists(return_path): return return_path
        return None

config_path = ConfigPath()



from os import listdir, makedirs
from os.path import isdir, join

from tornado.web import authenticated

from base import BaseHandler

class NotebookHandler(BaseHandler):
    @authenticated
    def get(self, notebook_name):
        notebook_name = notebook_name.replace('+', ' ')
        if not isdir(join(self.settings.repo, notebook_name)):
            self.redirect('/')
        else:
            if notebook_name.endswith('/'):
                notebook_name = notebook_name[:-1]
            path = join(self.settings.repo, notebook_name)
            notebook_contents = listdir(path)
            if '.git' in notebook_contents:
                notebook_contents.remove('.git')
            self.render('notebook.html', notebook_name=notebook_name,
                        note_name=None, notebook_contents=notebook_contents)

    @authenticated
    def post(self, notebook_name):
        path = join(self.settings.repo, notebook_name)
        makedirs(path)
        self.finish()



import logging
from os import path
from re import sub
from sh import find, grep, ErrorReturnCode_1
from urllib2 import unquote

from tornado.web import authenticated

from base import BaseHandler

class SearchHandler(BaseHandler):
    @authenticated
    def get(self):
        query = unquote(self.get_argument('q'))
        try:
            results = str(grep('-R', '--exclude-dir', '.git', query,
                               self.settings.repo))
        except ErrorReturnCode_1 as e:
            results = ''

        # TODO filter out duplicates if the filename is already in the search results
        # TODO this doesn't exclude the .git folder
        try:
            results += str(find(self.settings.repo, '-type', 'f', '-name',
                                '*' + query + '*'))
        except ErrorReturnCode_1 as e:
            pass

        results = results.replace(self.settings.repo, '').split('\n')[:-1]
        formatted_results = []
        for result in results:
            if 'Binary file' in result:
                continue

            # TODO this doesn't play well with colons in filenames
            stuff = result.split(':')
            filename = stuff[0]
            if path.basename(filename).startswith('.'):
                filename = path.join(path.dirname(filename),
                                     path.basename(filename)[1:])
            string = ''.join(stuff[1:])
            string = self._highlight(string, query)
            formatted_results.append({'filename': filename, 'string': string})
        self.render('search.html', query=query, results=formatted_results)



from index import IndexHandler
from config import ConfigHandler
from login import LoginHandler
from note import NoteHandler
from notebook import NotebookHandler
from search import SearchHandler

urls = []
urls.append((r'/?', IndexHandler))
urls.append((r'/config/?', ConfigHandler))
urls.append((r'/login/?', LoginHandler))
urls.append((r'/search/?', SearchHandler))

# do regex ones last so the others get routed properly
urls.append((r'/(.+)/(.+)', NoteHandler))
urls.append((r'/(.+)/?', NotebookHandler))



from os.path import join

import bcrypt
from tornado.web import authenticated

from base import BaseHandler

class ConfigHandler(BaseHandler):
    ALLOWED = {'testing': bool, 'port': int, 'pwdhash': str, 'repo': str,
               'username': str}
    @authenticated
    def get(self):
        self.render('config.html', config=self._fetch_existing_config())

    @authenticated
    def post(self):
        # TODO need confirm password field?
        old = self._fetch_existing_config()
        new = dict()
        for key in self.ALLOWED.keys():
            val = self.get_argument(key, None)
            if val is None or val == '':
                new[key] = old[key]
            elif key == 'pwdhash':
                new[key] = bcrypt.hashpw(val, bcrypt.gensalt())
            elif self.ALLOWED[key] == str:
                new[key] = str(val)
            elif self.ALLOWED[key] == int:
                new[key] = int(val)
            elif self.ALLOWED[key] == bool:
                new[key] = bool(val)
        config_file = open(self.settings.config_path.web, 'w')
        for key, val in new.items():
            if self.ALLOWED[key] == str:
                config_file.write("%s='%s'\n" % (key, val))
            else:
                config_file.write("%s=%s\n" % (key, val))
        config_file.close()
        self.redirect('/')

    def _fetch_existing_config(self):
        existing = dict()
        for config in open(self.settings.config_path.web).readlines():
            key, val = config.strip().replace(' = ', '=').split('=')
            existing[key] = val.replace("'", "")
        return existing



from os.path import isdir

from sh import ErrorReturnCode_128
from tornado.web import authenticated, RequestHandler

from base import BaseHandler

class IndexHandler(BaseHandler):
    @authenticated
    def get(self):
        config_problem = None
        if self.settings.repo is None:
            config_problem = {'type': 'danger',
                              'message': ('You have not configured the "repo" '
                                          'setting in magpie\'s config.')}
        elif not isdir(self.settings.repo):
            config_problem = {'type': 'danger',
                              'message': ('Your "repo" setting (%s) is not a '
                                          'valid directory' % \
                                          self.settings.repo)}
        else:
            try:
                self.application.git.status()
            except ErrorReturnCode_128 as e:
                if 'Not a git repository' in e.message:
                    config_problem = {'type': 'danger',
                                      'message': ('Your "repo" (%s) is not a '
                                                  'valid git repository' % \
                                                  self.settings.repo)}
                else:
                    raise

        if config_problem is None:
            if self.settings.username is None or self.settings.pwdhash is None \
            or self.settings.username == '' or self.settings.pwdhash == '':
                config_problem = {'type': 'warning',
                                  'message': ("It looks like you don't have a "
                                              "username and password set. It's "
                                              "not required, but it's "
                                              "recommended.")}
        self.render('index.html', config_problem=config_problem)



#!/usr/bin/env python
from os import path
from sys import argv

from pyPdf import PdfFileReader
from tornado.options import define, options, parse_config_file

from magpie.config import config_path

define('repo', default=None, type=str)
define('default_notebook', default='', type=str)
parse_config_file(config_path.pdf_scraper)

def scrape(file_path):
    pdf = PdfFileReader(file(file_path, 'rb'))
    content = ''
    for i in range(0, pdf.numPages):
        # TODO figure out why line breaks don't seem to be rendering
        content += pdf.getPage(i).extractText() + '\n'
    out_file = '.%s' % path.basename(file_path)
    out_path = path.join(path.dirname(file_path), out_file)
    if options.repo not in path.commonprefix([out_path, options.repo]):
        out_path = path.join(options.repo, options.default_notebook, out_file)
    content = content.encode('ascii', 'ignore')
    f = open(out_path, 'w')
    f.write(content)
    f.close()

if __name__ == '__main__':
    for file_name in argv[1:]:
        scrape(file_name)



from os import listdir
from os.path import isdir, join

from tornado.web import RequestHandler

class BaseHandler(RequestHandler):
    def render(self, template, **kwargs):
        if kwargs.get('notebook_name', None) is not None:
            path = join(self.settings.repo, kwargs['notebook_name'])
            kwargs['notes'] = sorted([n for n in listdir(path) if not \
                                      n.startswith('.')])
        else:
            kwargs['notebook_name'] = ''
            kwargs['notes'] = []
        if self.settings.repo is not None and isdir(self.settings.repo) and \
        not kwargs.get('hide_notebooks', False):
            kwargs['notebooks'] = sorted(listdir(self.settings.repo))
        else:
            kwargs['notebooks'] = []
        super(BaseHandler, self).render(template, **kwargs)

    def _highlight(self, text, highlight):
        return text.replace(highlight,
                            "<font style=background-color:yellow>%s</font>" % \
                            highlight)
    def get_current_user(self):
        if self.settings.username is None and self.settings.pwdhash is None \
        or self.settings.username == '' and self.settings.pwdhash == '':
            return True

        return self.get_cookie('session', '') == self.settings.session



#!/usr/bin/env python
from setuptools import setup

NAME = 'magpie'
DESCRIPTION = 'magpie: [ma]rkdown, [g]it, [pie]thon'

VERSION = open('VERSION').read().strip()
LONG_DESC = open('README.rst').read()
LICENSE = open('LICENSE').read()
REQUIREMENTS = [open('requirements.txt').readlines()]

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Intended Audience :: End Users/Desktop',
    'License :: OSI Approved :: MIT License',
    'Operating System :: MacOS :: MacOS X',
    'Operating System :: POSIX',
    'Programming Language :: Python :: 2.7',
]

PACKAGES=['magpie', 'magpie/config', 'magpie/handler', 'magpie/static',
          'magpie/template']
PACKAGE_DATA={'magpie': ['config/*.cfg', 'static/*', 'template/*.html']}
SCRIPTS=['utils/pdf_scraper.py', 'utils/email_notes.py']

setup(
    name=NAME,
    version=VERSION,
    author='Charles Thomas',
    author_email='ch@rlesthom.as',
    url='https://github.com/charlesthomas/%s' % NAME,
    license=LICENSE,
    description=DESCRIPTION,
    long_description=LONG_DESC,
    classifiers=CLASSIFIERS,
    install_requires=REQUIREMENTS,
    packages=PACKAGES,
    package_data=PACKAGE_DATA,
    scripts=SCRIPTS,
    entry_points='''
    [console_scripts]
    magpie = magpie.server:main
    ''',
)
